//
//  ViewController.m
//  LayerTest
//
//  Created by Thomas Anthony on 6/5/13.
//  Copyright (c) 2013 Thomas Anthony. All rights reserved.
//

#import "ViewController.h"
#import "CanvasView.h"
#import "QuartzCore/QuartzCore.h"

#define D2R(x) (x * (M_PI / 180.0f))

@interface ViewController ()

@end

@implementation ViewController
{
    CALayer *leftPageShadowLayer;
    CALayer *rightPageShadowLayer;
    CALayer *leftPage;
    CALayer *rightPage;
    UIView *curtainView;
}

- (void)loadView
{
    self.view = [[CanvasView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor blackColor];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    self.view.backgroundColor = [UIColor whiteColor];
    leftPageShadowLayer = [CAShapeLayer layer];
    rightPageShadowLayer = [CAShapeLayer layer];
    leftPageShadowLayer.anchorPoint = CGPointMake(1.0f, 0.5f);
    rightPageShadowLayer.anchorPoint = CGPointMake(0.0f, 0.5f);
    leftPageShadowLayer.position = CGPointMake(self.view.bounds.size.width / 2.0f,
                                               self.view.bounds.size.height / 2.0f);
    rightPageShadowLayer.position = CGPointMake(self.view.bounds.size.width / 2.0f,
                                                self.view.bounds.size.height / 2.0f);
    leftPageShadowLayer.bounds = CGRectMake(0,
                                            0,
                                            self.view.bounds.size.width / 2.0f,
                                            self.view.bounds.size.height);
    rightPageShadowLayer.bounds = CGRectMake(0,
                                             0,
                                             self.view.bounds.size.width / 2.0f,
                                             self.view.bounds.size.height);
    
    UIBezierPath *leftPageRoundedCornersPath = [UIBezierPath bezierPathWithRoundedRect:leftPageShadowLayer.bounds byRoundingCorners:UIRectCornerTopLeft|UIRectCornerBottomLeft cornerRadii:CGSizeMake(25.0f, 25.0f)];
    UIBezierPath *rightPageRoundedCornersPath = [UIBezierPath bezierPathWithRoundedRect:rightPageShadowLayer.bounds byRoundingCorners:UIRectCornerTopRight|UIRectCornerBottomRight cornerRadii:CGSizeMake(25.0f, 25.0f)];
    leftPageShadowLayer.shadowPath = leftPageRoundedCornersPath.CGPath;
    rightPageShadowLayer.shadowPath = rightPageRoundedCornersPath.CGPath;
    leftPageShadowLayer.shadowRadius = 100.0f;
    leftPageShadowLayer.shadowColor = [UIColor blackColor].CGColor;
    leftPageShadowLayer.shadowOpacity = 0.9f;
    rightPageShadowLayer.shadowRadius = 100.0f;
    rightPageShadowLayer.shadowColor = [UIColor blackColor].CGColor;
    rightPageShadowLayer.shadowOpacity = 0.9f;
    
    leftPage = [CALayer layer];
    rightPage = [CALayer layer];
    leftPage.frame = leftPageShadowLayer.bounds;
    rightPage.frame = rightPageShadowLayer.bounds;
    leftPage.backgroundColor = [UIColor whiteColor].CGColor;
    rightPage.backgroundColor = [UIColor whiteColor].CGColor;
    leftPage.transform = makePerspectiveTransform();
    rightPage.transform = makePerspectiveTransform();
    
    CAShapeLayer *leftPageRoundedCornersMask = [CAShapeLayer layer];
    CAShapeLayer *rightPageRoundedCornersMask = [CAShapeLayer layer];
    leftPageRoundedCornersMask.frame = leftPage.bounds;
    rightPageRoundedCornersMask.frame = rightPage.bounds;
    leftPageRoundedCornersMask.path = leftPageRoundedCornersPath.CGPath;
    rightPageRoundedCornersMask.path = rightPageRoundedCornersPath.CGPath;
    leftPage.mask = leftPageRoundedCornersMask;
    rightPage.mask = rightPageRoundedCornersMask;
    leftPageShadowLayer.transform = makePerspectiveTransform();
    rightPageShadowLayer.transform = makePerspectiveTransform();
    
    curtainView = [[UIView alloc] initWithFrame:self.view.bounds];
    curtainView.backgroundColor = [UIColor scrollViewTexturedBackgroundColor];
    [curtainView.layer addSublayer:leftPageShadowLayer];
    [curtainView.layer addSublayer:rightPageShadowLayer];
    [leftPageShadowLayer addSublayer:leftPage];
    [rightPageShadowLayer addSublayer:rightPage];
    
    UIPinchGestureRecognizer *pinch = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(foldWithPinch:)];
    [self.view addGestureRecognizer:pinch];
}

CATransform3D makePerspectiveTransform()
{
    CATransform3D transform = CATransform3DIdentity;
    transform.m34 = 1.0f / -2000;
    return transform;
}

- (void)foldWithPinch:(UIPinchGestureRecognizer *)pinch
{
    if (pinch.state == UIGestureRecognizerStateBegan)
    {
        self.view.userInteractionEnabled = NO;
        CGImageRef imageRef = ((CanvasView *)self.view).image.CGImage;
        leftPage.contents = (__bridge id)imageRef;
        rightPage.contents = (__bridge id)imageRef;
        leftPage.contentsRect = CGRectMake(0.0f, 0.0f, 0.5f, 1.0f);
        rightPage.contentsRect = CGRectMake(0.5f, 0.0f, 0.5f, 1.0f);
        leftPageShadowLayer.transform = CATransform3DIdentity;
        rightPageShadowLayer.transform = CATransform3DIdentity;
        leftPageShadowLayer.transform = makePerspectiveTransform();
        rightPageShadowLayer.transform = makePerspectiveTransform();
        [self.view addSubview:curtainView];
    }
    float scale = pinch.scale > 0.5f ? pinch.scale : 0.5f;
    scale = scale < 1.0f ? scale : 1.0f;
    [CATransaction begin];
    [CATransaction setDisableActions:YES];
    [CATransaction commit];
    leftPageShadowLayer.transform = CATransform3DIdentity;
    rightPageShadowLayer.transform = CATransform3DIdentity;
    leftPageShadowLayer.transform = makePerspectiveTransform();
    rightPageShadowLayer.transform = makePerspectiveTransform();
    leftPageShadowLayer.transform = CATransform3DScale(leftPageShadowLayer.transform,
                                                       scale,
                                                       scale,
                                                       scale);
    rightPageShadowLayer.transform = CATransform3DScale(rightPageShadowLayer.transform,
                                                        scale,
                                                        scale,
                                                        scale);
    leftPageShadowLayer.transform = CATransform3DRotate(leftPageShadowLayer.transform,
                                                        (1.0 - scale) * M_PI,
                                                        0.0f,
                                                        1.0f,
                                                        0.0f);
    rightPageShadowLayer.transform = CATransform3DRotate(rightPageShadowLayer.transform,
                                                         -(1.0 - scale) * M_PI,
                                                         0.0f,
                                                         1.0f,
                                                         0.0f);
    
    if (pinch.state == UIGestureRecognizerStateEnded)
    {
        [CATransaction begin];
        [CATransaction setCompletionBlock:^{
            self.view.userInteractionEnabled = YES;
            [curtainView removeFromSuperview];
        }];
        // Set the animation duration inversely proportional to the scale of the user's
        // pinch gesture
        [CATransaction setAnimationDuration:0.5f/scale];
        leftPageShadowLayer.transform = CATransform3DIdentity;
        rightPageShadowLayer.transform = CATransform3DIdentity;
        [CATransaction commit];
    }
}

@end
